from django.contrib import admin

from pennyblack import settings
from pennyblack.models.newsletter import Newsletter, NewsletterAdmin
from pennyblack.models.job import Job, JobAdmin, JobStatistic, JobStatisticAdmin
from pennyblack.models.sender import Sender, SenderAdmin

admin.site.register(Newsletter, NewsletterAdmin)
admin.site.register(Job,JobAdmin)
admin.site.register(JobStatistic, JobStatisticAdmin)
admin.site.register(Sender,SenderAdmin)


from django.conf import settings

TINYMCE_CONFIG_URL = getattr(settings, 'PENNYBLACK_TINYMCE_CONFIG_URL', 'admin/pennyblack/tiny_mce/init.html')

LANGUAGES = getattr(settings, 'LANGUAGES')
LANGUAGE_CODE = getattr(settings, 'LANGUAGE_CODE')

NEWSLETTER_TYPE = getattr(settings, 'PENNYPLACK_NEWLETTER_TYPE', ((1, 'Massmail'),(2, 'Workflow')))
NEWSLETTER_TYPE_MASSMAIL = getattr(settings, 'PENNYPLACK_NEWLETTER_TYPE_MASSMAIL', (1,))
NEWSLETTER_TYPE_WORKFLOW = getattr(settings, 'PENNYPLACK_NEWLETTER_TYPE_WORKFLOW', (2,))

JOB_STATUS = getattr(settings, 'PENNYBLACK_JOB_STATUS', ((1,'Draft'),(11,'Pending'),(21,'Sending'),(31,'Finished'),(41,'Error'),(42,'Timeout (will retry)'),(32,'ReadOnly')))

JOB_STATUS_CAN_SEND = getattr(settings, 'PENNYBLACK_JOB_STATUS_CAN_SEND', (1,41))
JOB_STATUS_PENDING = getattr(settings, 'PENNYBLACK_JOB_STATUS_PENDING', (11,42))
JOB_STATUS_CAN_EDIT = getattr(settings, 'PENNYBLACK_JOB_STATUS_CAN_EDIT', (1,))
JOB_MAIL_INLINE_COUNT = getattr(settings, 'PENNYBLACK_JOB_MAIL_INLINE_COUNT', 50)
# bounce detection
BOUNCE_DETECTION_ENABLE = getattr(settings, 'PENNYBLACK_BOUNCE_DETECTION_ENABLE', False)
BOUNCE_DETECTION_DAYS_TO_LOOK_BACK = getattr(settings, 'PENNYBLACK_BOUNCE_DETECTION_DAYS_TO_LOOK_BACK',5)
BOUNCE_DETECTION_BOUNCE_EMAIL_FOLDER = getattr(settings, 'PENNYBLACK_BOUNCE_DETECTION_BOUNCE_EMAIL_FOLDER', 'INBOX.bounced')

# content
NEWSLETTER_CONTENT_WIDTH = getattr(settings, 'PENNYBLACK_NEWSLETTER_CONTENT_WIDTH', 600)

TEXT_AND_IMAGE_CONTENT_POSITIONS = getattr(settings, 'PENNYBLACK_TEXT_AND_IMAGE_CONTENT_POSITIONS', (('left','Left'),('right','Right'),('top','Top')))
TEXT_AND_IMAGE_CONTENT_IMAGE_WIDTH_SIDE = getattr(settings, 'PENNYBLACK_TEXT_AND_IMAGE_CONTENT_IMAGE_WIDTH_SIDE', 100)

JPEG_QUALITY = getattr(settings, 'PENNYBLACK_JPEG_QUALITY', 75)

# subscriber module

SUBSCRIBER_BOUNCES_UNTIL_DEACTIVATION = getattr(settings, 'SUBSCRIBER_BOUNCES_UNTIL_DEACTIVATION', 2)


from django.contrib import admin
from django.core.mail.utils import DNS_NAME
from django.db import models
from django.utils.translation import ugettext_lazy as _

from pennyblack import settings

if settings.BOUNCE_DETECTION_ENABLE:
    from Mailman.Bouncers.BouncerAPI import ScanText

import imaplib
import datetime
import socket
try:
    import spf
    ENABLE_SPF = True
except IOError:
    # spf fails to load on a system which is offline because of missing resolv.conf
    ENABLE_SPF = False
except ImportError:
    # spf missing
    ENABLE_SPF = False
    

#-----------------------------------------------------------------------------
# Sender
#-----------------------------------------------------------------------------
class Sender(models.Model):
    """
    A sender for the from and reply to fields of the newsletter.
    """
    email = models.EmailField(verbose_name=_("from e-mail address"))
    name = models.CharField(verbose_name=_("from name"), help_text=_("many e-mail clients show this as from."), max_length=100)
    imap_username = models.CharField(verbose_name=_("imap username"), max_length=100, blank=True)
    imap_password = models.CharField(verbose_name=_("imap passwort"), max_length=100, blank=True)
    imap_server = models.CharField(verbose_name=_("imap server"), max_length=100, blank=True)
    imap_port = models.IntegerField(verbose_name=_("imap port"), max_length=100, default=143)
    imap_ssl = models.BooleanField(verbose_name=_("use ssl"), default=False)
    get_bounce_emails = models.BooleanField(verbose_name=_("get bounce e-mails"), default=False)
    
    class Meta:
        verbose_name = _('sender')
        verbose_name_plural = _('senders')
        app_label = 'pennyblack'
    
    def __unicode__(self):
        return self.email
    
    def check_spf(self):
        """
        Check if sender is authorised by sender policy framework
        """
        if not ENABLE_SPF:
            return False
        return spf.check(i=socket.gethostbyname(DNS_NAME.get_fqdn()),s=self.email,h=DNS_NAME.get_fqdn())
    
    def spf_result(self):
        return self.check_spf()
    check_spf.short_description = "spf Result"
    
    def get_mail(self):
        """
        Checks the inbox of this sender and prcesses the bounced emails
        """
        from pennyblack.models import Mail
        if not settings.BOUNCE_DETECTION_ENABLE:
            return
        oldest_date = datetime.datetime.now()-datetime.timedelta(days=settings.BOUNCE_DETECTION_DAYS_TO_LOOK_BACK)
        try:
            if self.imap_ssl:
                ssl_class = imaplib.IMAP4_SSL
            else:
                ssl_class = imaplib.IMAP4
            conn = ssl_class(self.imap_server, int(self.imap_port))
            conn.login(self.imap_username, self.imap_password)
            if conn.select(settings.BOUNCE_DETECTION_BOUNCE_EMAIL_FOLDER)[0] != 'OK':
                conn.create(settings.BOUNCE_DETECTION_BOUNCE_EMAIL_FOLDER)
            conn.select('INBOX')
            typ, data = conn.search(None, 'ALL')
            for num in data[0].split():
                typ, data = conn.fetch(num, '(RFC822)')
                if not data or not data[0]:
                    continue
                addrs = ScanText(data[0][1])
                addrs = addrs.split(';')
                if len(addrs) == 1 and len(addrs[0]) == 0:
                    continue
                for addr in addrs:
                    mailquery = Mail.objects.filter(email=addr).filter(job__date_deliver_finished__gte=oldest_date)
                    # ping all newsletter receivers
                    for mail in mailquery:
                        mail.bounce()
                if conn.copy(num,settings.BOUNCE_DETECTION_BOUNCE_EMAIL_FOLDER)[0] == 'OK':
                    conn.store(num, '+FLAGS', r'\Deleted')
            conn.expunge()
            conn.close()
            conn.logout()
        except imaplib.IMAP4.error, e:
            return
        
class SenderAdmin(admin.ModelAdmin):
    list_display = ('email', 'name',)
    fields = ('email', 'name', 'imap_username', 'imap_password', 'imap_server', 'imap_port', 'imap_ssl', 'get_bounce_emails', 'spf_result',)
    readonly_fields = ('spf_result',)




from django.contrib import admin
from django.contrib.contenttypes import generic
from django.db import models

from pennyblack import settings
from pennyblack.options import NewsletterReceiverMixin, JobUnitMixin, JobUnitAdmin

import datetime

class NewsletterSubscriberManager(models.Manager):
    """
    Custom manager for NewsletterSubscriber to provide extra functionality
    """
    use_for_related_fields = True
    
    def get_or_add(self,email, **kwargs):
        """
        Gets a subscriber, if he doesn't exist it creates him.
        """
        try:
            return self.get(email__iexact=email)
        except self.model.DoesNotExist:
            return self.create(email=email.lower(), **kwargs)
    
    def active(self):
        """
        Gives only the active subscribers
        """
        return self.filter(is_active=True)
        
newsletter_subscriber_manager = NewsletterSubscriberManager()

class NewsletterSubscriber(models.Model, NewsletterReceiverMixin):
    """
    A generic newsletter subscriber
    """
    # from pennyblack.models.mail import Mail
    email = models.EmailField(verbose_name="email address", unique=True)
    groups = models.ManyToManyField('subscriber.SubscriberGroup',
        verbose_name="Groups", related_name='subscribers')
    date_subscribed = models.DateTimeField(verbose_name="Subscribe Date",
        default=datetime.datetime.now())
    mails = generic.GenericRelation('pennyblack.Mail')
    is_active = models.BooleanField(verbose_name="Active", default=True)
    
    objects = newsletter_subscriber_manager
    default_manager = newsletter_subscriber_manager
    
    class Meta:
        verbose_name = "Subscriber"
        verbose_name_plural = "Subscribers"
    
    def __unicode__(self):
        return self.email
    
    def on_bounce(self,mail):
        """
        A mail got bounced, consider deactivating this subscriber.
        """
        bounce_count = 0
        for mail in self.mails.order_by('pk'):
            bounce_count += mail.bounced
            if mail.viewed:
                bounce_count = 0
        if bounce_count >= settings.SUBSCRIBER_BOUNCES_UNTIL_DEACTIVATION:
            self.is_active = False
            self.save()
    
    def unsubscribe(self):
        self.is_active = False
        self.save()
    
    @classmethod
    def register_extension(cls, register_fn):
        """
        Call the register function of an extension. You must override this
        if you provide a custom ModelAdmin class and want your extensions to
        be able to patch stuff in.
        """
        register_fn(cls, NewsletterSubscriberAdmin)


class NewsletterSubscriberAdmin(admin.ModelAdmin):
    search_fields = ('email',)
    list_filter = ('groups', 'is_active')
    list_display = ('__unicode__','is_active')
    filter_horizontal = ('groups',)
    

class SubscriberGroupManager(models.Manager):
    """
    Custom manager for SubscriberGroup to provide extra functionality
    """
    def get_or_add(self, name, **kwargs):
        """
        Gets a group, if she doesn't exist it creates her.
        """
        try:
            return self.get(name__iexact=name)
        except self.model.DoesNotExist:
            return self.create(name=name, **kwargs)
    
class SubscriberGroup(models.Model, JobUnitMixin):
    """
    Groups to add newsletter subscribers
    """
    name = models.CharField(max_length=50, verbose_name="Name", unique=True)
    
    objects = SubscriberGroupManager()

    class Meta:
        verbose_name = "Subscriber Group"
        verbose_name_plural = "Subscriber Groups"
    
    def __unicode__(self):
        return self.name
    
    @property
    def member_count(self):
        return self.subscribers.active().count()
    
    def get_member_count(self):
        return self.member_count
    get_member_count.short_description = "Member Count"
    
    def get_newsletter_receiver_collections(self):
        """
        Every Group has only one collection
        """
        return (('all',{}),)
    
    def get_receiver_queryset(self):
        """
        Return all group members
        """
        return self.subscribers.active()
    
class SubscriberGroupAdmin(JobUnitAdmin):
    list_display = ('__unicode__', 'get_member_count')

# register view links
from pennyblack.models import Newsletter
from pennyblack.module.subscriber.views import unsubscribe
Newsletter.register_view_link('subscriber.unsubscribe',unsubscribe)



from django.contrib.auth.decorators import login_required
from django.core.exceptions import ObjectDoesNotExist
from django.core.urlresolvers import reverse
from django.db.models import F
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from django.utils.functional import wraps

from pennyblack.models import Newsletter, Link, Mail, Job

import types

def needs_mail(function):
    """
    Decorator to get the mail object
    """
    @wraps(function)
    def wrapper(request, mail_hash=None, *args, **kwargs):
        try:
            mail = Mail.objects.get(mail_hash=mail_hash)
        except ObjectDoesNotExist:
            return HttpResponseRedirect('/')
        return function(request, mail=mail, *args, **kwargs)
    return wrapper

def needs_link(function):
    """
    Decorator to get the link object
    """
    @wraps(function)
    def wrapper(request, link_hash=None, *args, **kwargs):
        try:
            link = Link.objects.get(link_hash=link_hash)
        except ObjectDoesNotExist:
            return HttpResponseRedirect('/')
        return function(request, link=link, *args, **kwargs)
    return wrapper
            

@login_required
def preview(request, newsletter_id):
    """
    Previews this newsletter as a Website
    """
    newsletter = Newsletter.objects.filter(pk=newsletter_id)[0]
    request.content_context = {
        'newsletter' : newsletter,
        'webview' : True,
        }
    job_id = request.GET.get('job',None)
    if job_id:
        job = get_object_or_404(Job, pk=job_id)
        if job.mails.count():
            mail = job.mails.all()[0]
            request.content_context.update(mail.get_context())
            request.content_context.update({'base_url':''})
    return render_to_response(newsletter.template.path, request.content_context, context_instance=RequestContext(request))
    
@needs_mail
@needs_link
def redirect_link(request, mail, link):
    """
    Redirects to the link target and marks this mail as read
    """
    mail.on_landing(request)
    target = link.click(mail)
    if isinstance(target, types.FunctionType):
        return HttpResponseRedirect(reverse('pennyblack.proxy', args=(mail.mail_hash, link.link_hash)))
    ga_tracking = "utm_source=%s&utm_medium=%s&utm_campaign=%s" % (
        mail.job.newsletter.utm_source, mail.job.newsletter.utm_medium,
        mail.job.utm_campaign)
    if target.find('?') > 0:
        target = '%s&%s' % (target, ga_tracking)
    else:
        target = '%s?%s' % (target, ga_tracking)
    return HttpResponseRedirect(target)
    
@needs_mail
def ping(request, mail, filename):
    mail.mark_viewed()
    return HttpResponseRedirect(mail.job.newsletter.header_image.get_absolute_url())


@needs_mail
def view(request, mail):
    mail.mark_viewed()
    return HttpResponse(mail.get_content(webview=True))

@needs_mail
@needs_link
def proxy(request, mail, link):
    return link.get_target(mail)(request, mail.person, mail.job.group_object)



from pennyblack.models import Newsletter, Mail
from pennyblack.options import NewsletterReceiverMixin
from pennyblack.content.richtext import TextOnlyNewsletterContent
from django.db import models
from django.core.urlresolvers import reverse
import unittest

class NewsletterTestCase(unittest.TestCase):
    # def setUp(self):
    #     pass
        
    def test_is_valid(self):
        n = Newsletter()
        self.assertFalse(n.is_valid(), "Empty Newsletter can not be valid.")
        n.subject = 'some text'
        self.assertTrue(n.is_valid(), "Newsletter should be valid now.")

# class MailTest(unittest.TestCase):
#     receiver = None
#     mail = None
#     def setUp(self):
#         receiver = NewsletterReceiver()
#         mail = Mail(person=self.receiver)
#     
#     def test_is_valid(self):
#         self.assertFalse(self.mail.is_valid())

class RichtextContentTest(unittest.TestCase):
    content = None
    job = None
    class Job(object):
        def __init__(self, link):
            self.times = 0
            self.link = link
        def add_link(self, link):
            self.times += 1
            return '{{base_url}}' + self.link

    def setUp(self):
        self.content = TextOnlyNewsletterContent(text='<a href="http://www.test.com">link</a>')
        self.link = reverse('pennyblack.redirect_link', kwargs={'mail_hash':'{{mail.mail_hash}}','link_hash':'1234'}).replace('%7B','{').replace('%7D','}')
        self.job = self.Job(self.link)
        
    
    def test_replace_links(self):
        self.content.replace_links(self.job)
        self.assertEqual(self.content.text,'<a href="{{base_url}}%s">link</a>' % self.link)

    def test_replace_multiple_links(self):
        self.content.text='<a href="http://www.testmultiple.com">link</a><a href="http://www.2ndlink.com">2ndlink</a>'
        self.content.replace_links(self.job)
        self.assertEqual(self.content.text,'<a href="{{base_url}}%s">link</a><a href="{{base_url}}%s">2ndlink</a>' % (self.link, self.link))
    
    def test_dont_replace_twice(self):
        self.content.text = '<a href="http://www.allink.ch">link</a>'
        self.content.replace_links(self.job)
        old_times = self.job.times
        last_text = self.content.text[:]
        self.content.replace_links(self.job)
        self.assertEqual(self.job.times, old_times)
        self.assertEqual(self.content.text, last_text)
    
    def test_dont_replace_link_url_tag_urls(self):
        link = '<a href="{% link_url my_identifier%}">link</a>'
        self.content.text = link
        self.content.replace_links(self.job)
        self.assertEqual(self.content.text,link)
    
    def test_quotes_in_url(self):
        self.content.text = '<a href="http://test{{somevariable|with:"quotes"}}">link</a>'
        self.content.replace_links(self.job)
        self.assertEqual(self.content.text,'<a href="{{base_url}}%s">link</a>' % self.link)
    
    def test_link_style(self):
        self.content.text = '<a >link</a>'
        self.content.prepare_to_send()
        self.assertEqual(self.content.text,'<a {% get_newsletterstyle request text_and_image_title %}>link</a>')

    def test_multiple_link_styles(self):
        self.content.text = '<a >link</a><a >link</a>'
        self.content.prepare_to_send()
        self.assertEqual(self.content.text,'<a {% get_newsletterstyle request text_and_image_title %}>link</a><a {% get_newsletterstyle request text_and_image_title %}>link</a>')



from django.contrib import admin
from django.core.context_processors import csrf
from django.core.urlresolvers import reverse
from django.db import models
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404, render_to_response

from pennyblack.forms import CollectionSelectForm

import exceptions

class NewsletterReceiverMixin(object):
    """
    Mixin for every object that can receive a newsletter
    """    
    def get_email(self):
        if hasattr(self,'email'):
            return self.email
        raise exceptions.NotImplementedError('Need a get_email implementation.')
    
    def on_bounce(self, mail):
        pass
        
class JobUnitMixin(object):
    """
    Mixin for every object which can be target of a Job
    """    
    def create_newsletter(self, form_data=None):
        """
        Creates a newsletter for every NewsletterReceiverMixin
        """
        from pennyblack.models.job import Job
        if form_data==None:
            collection_name = 'Default'
            queryset = self.get_receiver_queryset()
        else:
            collection_name = ', '.join(form_data['collections'])
            queryset = self.get_receiver_filtered_queryset(**form_data)
        job = Job(group_object=self, collection=collection_name)
        job.save()
        job.create_mails(queryset)
        return job
        
    def get_newsletter_receiver_collections(self):
        """
        Returns a dict of valid receiver collections
        has to be overriden in the object to return a tuple of querysets
        return (('all',{}),)
        {} is used to filter the queryset when evaluating the
        get_receiver_filtered_queryset function.
        """
        raise exceptions.NotImplementedError("Override this method in your class!")
    
    def get_receiver_queryset(self):
        raise exceptions.NotImplementedError("Override this method in your class!")
    
    def get_receiver_filtered_queryset(self, **kwargs):
        """
        Takes the second part of the collections 
        """
        queryset = self.get_receiver_queryset()
        return queryset


class JobUnitAdmin(admin.ModelAdmin):
    """
    Admin model for objects wich are capable of sending newsletters to it's
    members.
    """
    change_form_template = "admin/pennyblack/jobunit/change_form.html"
    collection_select_form = CollectionSelectForm
    collection_selection_form_extra_fields = dict()
    
    def create_newsletter(self, request, object_id):
        obj = get_object_or_404(self.model, pk=object_id)
        if len(obj.get_newsletter_receiver_collections()) == 1 and len(self.collection_selection_form_extra_fields) == 0:
            # there is only one collection and no options to select
            # -> call create_newsletter directly
            job = obj.create_newsletter()
            return HttpResponseRedirect(reverse('admin:pennyblack_job_change', args=(job.id,)))            
        if request.method == 'POST':
            form = self.collection_select_form(data=request.POST,
                group_object=obj,
                extra_fields=self.collection_selection_form_extra_fields)
            if form.is_valid():
                job = obj.create_newsletter(form_data=form.cleaned_data)
                return HttpResponseRedirect(reverse('admin:pennyblack_job_change', args=(job.id,)))
        else:
            form = self.collection_select_form(group_object=obj,
                extra_fields=self.collection_selection_form_extra_fields)
        info = self.model._meta.app_label, self.model._meta.module_name
        context = {
            'opts':self.model._meta,
            'app_label':self.model._meta.app_label,
            'adminform':form,
            'form_url' : reverse('admin:%s_%s_create_newsletter' % info, args=(obj.id,))
        }
        context.update(csrf(request))
        return render_to_response('admin/pennyblack/jobunit/select_receiver_collection.html',context)
            
    def get_urls(self):
        from django.conf.urls.defaults import patterns, url
        urls = super(JobUnitAdmin, self).get_urls()
        info = self.model._meta.app_label, self.model._meta.module_name
        my_urls = patterns('',
            url(r'^(?P<object_id>\d+)/create_newsletter/$', self.admin_site.admin_view(self.create_newsletter), name='%s_%s_create_newsletter' % info),
        )
        return my_urls + urls



from django.contrib import admin
from django.contrib.contenttypes import generic
from django.core import mail
from django.core.validators import email_re
from django.db import models
from django.http import HttpRequest
from django.template.loader import render_to_string
from django.template import loader, Context, Template, RequestContext

import datetime
import hashlib
import random

from pennyblack import settings

#-----------------------------------------------------------------------------
# Mail
#-----------------------------------------------------------------------------
class Mail(models.Model):
    """
    This is a single Mail, it's part of a Job
    """
    viewed = models.DateTimeField(default=None, null=True)
    bounced = models.BooleanField(default=False)
    sent = models.BooleanField(default=False)
    content_type = models.ForeignKey('contenttypes.ContentType')
    object_id = models.PositiveIntegerField()
    person = generic.GenericForeignKey('content_type', 'object_id')
    job = models.ForeignKey('pennyblack.Job', related_name="mails")
    mail_hash = models.CharField(max_length=32, blank=True)
    email = models.EmailField() # the address is stored when the mail is sent
    
    class Meta:
        verbose_name = 'mail'
        verbose_name_plural = 'mails'
        app_label = 'pennyblack'
    
    def __unicode__(self):
        return u'%s to %s' % (self.job, self.person,)
        
    def save(self, **kwargs):
        if self.mail_hash == u'':
            self.mail_hash = hashlib.md5(str(self.id)+str(random.random())).hexdigest()
        super(Mail, self).save(**kwargs)
    
    def mark_sent(self):
        """
        Marks the email as beeing sent.
        """
        self.sent = True
        self.save()
    
    def mark_viewed(self):
        """
        Marks the email as beeing viewed and if it's not already viewed it
        stores the view date.
        """
        if not self.viewed:
            self.viewed = datetime.datetime.now()
            self.save()
    
    def on_landing(self, request):
        """
        Is executed every time a user landed on the website after clicking on
        a link in this email. It tries to execute the on_landing method on the
        person object and on the group object.
        """
        self.mark_viewed()
        if hasattr(self.person, 'on_landing') and hasattr(self.person.on_landing, '__call__'):
            self.person.on_landing(request)
        if self.job.content_type is not None and \
            hasattr(self.job.group_object, 'on_landing') and \
            hasattr(self.job.group_object.on_landing, '__call__'):
            self.group_object.on_landing(request)
    
    def bounce(self):
        """
        Is executed if this email is bounced.
        """
        self.bounced = True
        self.save()
        self.person.on_bounce(self)
        
    def unsubscribe(self):
        """
        Is executed if the unsubscribe link is clicked.
        """
        return self.person.unsubscribe()
    
    def is_valid(self):
        """
        Checks if this Mail is valid by validating the email address.
        """
        return email_re.match(self.person.get_email())

    def get_email(self):
        """
        Gets the email address. If it has no email address set, it tries to
        get it from the person object.
        """
        if self.email != '':
            return self.email
        return self.person.get_email()
    get_email.short_description = "E-Mail"

    def get_message(self):
        """
        Returns a email message object
        """
        self.email = self.person.get_email()
        if self.job.newsletter.reply_email!='':
            headers={'Reply-To': self.job.newsletter.reply_email}
        else:
            headers={}
        message = mail.EmailMessage(
            self.job.newsletter.subject,
            self.get_content(),
            self.job.newsletter.sender.email,
            [self.email],
            headers=headers,
        )
        message.content_subtype = "html"
        return message
    
    def get_content(self, webview=False):
        """
        Renders the email content. If webview is True it includes also a
        html header and doesn't display the webview link.
        """
        newsletter = self.job.newsletter
        context = self.get_context()
        context['newsletter'] = newsletter
        context['webview'] = webview
        request = HttpRequest()
        request.content_context = context
        return render_to_string(newsletter.template.path,
            context, context_instance=RequestContext(request))
        
    
    def get_context(self):
        """
        Returns the context of this email as a dict.
        """        
        return {
            'person': self.person,
            'group_object': self.job.group_object,
            'mail':self,
            'base_url': self.job.newsletter.get_base_url()
        }
    
    def get_header_url(self):
        """
        Gets the header url for this email.
        """
        return self.job.newsletter.header_url_replaced.replace('{{mail.mail_hash}}',self.mail_hash).replace('{{base_url}}', self.job.newsletter.get_base_url())

class MailInline(admin.TabularInline):
    model = Mail
    max_num = 0
    can_delete = False
    fields = ('get_email',)
    readonly_fields = ('get_email',)
    
    def queryset(self, request):
        """
        Don't display Inlines if there are more than a certain amount
        """
        if request._pennyblack_job_obj.mails.count() > settings.JOB_MAIL_INLINE_COUNT:
            return super(MailInline,self).queryset(request).filter(pk=0)
        return super(MailInline,self).queryset(request)



from django.contrib import admin

from pennyblack.module.subscriber.models import NewsletterSubscriber,\
    NewsletterSubscriberAdmin, SubscriberGroup, SubscriberGroupAdmin

admin.site.register(NewsletterSubscriber, NewsletterSubscriberAdmin)
admin.site.register(SubscriberGroup, SubscriberGroupAdmin)


from django import forms

class CollectionSelectForm(forms.Form):
    collections = forms.MultipleChoiceField()
    def __init__(self, group_object=None, extra_fields=None, *args, **kwargs):
       super(CollectionSelectForm, self).__init__(*args, **kwargs)
       choices = tuple((number,c[0]) for number, c in enumerate(group_object.get_newsletter_receiver_collections()))
       self.fields['collections'].choices = choices
       for key, field in extra_fields.items():
           self.fields[key] = field



VERSION = (0, 3, 5,)
__version__ = '.'.join(map(str, VERSION))

# Do not use Django settings at module level as recommended
try:
    from django.utils.functional import LazyObject
except ImportError:
    pass
else:
    class LazySettings(LazyObject):
        def _setup(self):
            from pennyblack import default_settings
            self._wrapped = Settings(default_settings)

    class Settings(object):
        def __init__(self, settings_module):
            for setting in dir(settings_module):
                if setting == setting.upper():
                    setattr(self, setting, getattr(settings_module, setting))

    settings = LazySettings()

def send_newsletter(newsletter_name, *args, **kwargs):
    """
    Gets a newsletter by its name and tries to send it to receiver
    """
    from pennyblack.models import Newsletter
    newsletter = Newsletter.objects.get_workflow_newsletter_by_name(newsletter_name)
    if newsletter:
        newsletter.send(*args, **kwargs)


from django.core import files
from django.db import models
from django.forms.util import ErrorList
from django.template import Context, Template, TemplateSyntaxError
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _

from pennyblack import settings
from pennyblack.models.link import check_if_redirect_url, is_link

from feincms.content.richtext.models import RichTextContentAdminForm, RichTextContent
from feincms.module.medialibrary.models import MediaFile

import re
import os
import Image
import exceptions

HREF_RE = re.compile(r'href\="((\{\{[^}]+\}\}|[^"><])+)"')

class NewsletterSectionAdminForm(RichTextContentAdminForm):
    def clean(self):
        cleaned_data = super(NewsletterSectionAdminForm, self).clean()
        try:
            t = Template(cleaned_data['text'])
        except TemplateSyntaxError, e:
            self._errors["text"] = ErrorList([e])
        except exceptions.KeyError:
            pass
        try:
            t = Template(cleaned_data['title'])
        except TemplateSyntaxError, e:
            self._errors["title"] = ErrorList([e])
        except exceptions.KeyError:
            pass
        return cleaned_data
    
    class Meta:
        exclude = ('image_thumb', 'image_width', 'image_height', 'image_url_replaced')
    
    def __init__(self, *args, **kwargs):
        super(NewsletterSectionAdminForm, self).__init__(*args, **kwargs)
        self.fields.insert(0, 'title', self.fields.pop('title'))

class TextOnlyNewsletterContent(RichTextContent):
    """
    Has a title and a text wich both can contain template code.
    """
    title = models.CharField(max_length=500)
    form = NewsletterSectionAdminForm
    feincms_item_editor_form = NewsletterSectionAdminForm

    feincms_item_editor_includes = {
        'head': [ settings.TINYMCE_CONFIG_URL ],
        }
    
    baselayout = "content/text_only/section.html"

    class Meta:
        abstract = True
        verbose_name = _('text only content')
        verbose_name_plural = _('text only contents')

    def replace_links(self, job):
        """
        Replaces all links and inserts pingback links
        """
        offset = 0
        for match in HREF_RE.finditer(self.text):
            link = match.group(1)
            if check_if_redirect_url(link):
                continue
            # don't replace links to proxy view
            if u'link_url' in link:
                continue
            replacelink = job.add_link(link)
            self.text = ''.join((self.text[:match.start(1)+offset], replacelink, self.text[match.end(1)+offset:]))
            offset += len(replacelink) - len(match.group(1))
        
    def prepare_to_send(self):
        """
        insert link_style into all a tags
        """
        self.text = re.sub(r"<a ","<a style=\"{% get_newsletterstyle request link_style %}\"", self.text)
        self.save()
        
    def get_template(self):
        """
        Creates a template
        """
        return Template("""{%% extends "%s" %%}
        {%% load pennyblack_tags %%}
        {%% block title %%}%s{%% endblock %%}
        {%% block text %%}%s{%% endblock %%}
        """ % (self.baselayout, self.title, self.text,))
    
    def render(self, request, **kwargs):
        context = request.content_context
        context['request'] = request
        context.update({'content':self, 'content_width':settings.NEWSLETTER_CONTENT_WIDTH})
        if hasattr(self,'get_extra_context'):
            context.update(self.get_extra_context())
        return self.get_template().render(Context(context))


class TextWithImageNewsletterContent(TextOnlyNewsletterContent):
    """
    Like a TextOnlyNewsletterContent but with extra image field
    """
    image_original = models.ForeignKey(MediaFile)
    image_thumb = models.ImageField(upload_to='newsletter/images', blank=True,
        width_field='image_width', height_field='image_height')
    image_width = models.IntegerField(default=0)
    image_height = models.IntegerField(default=0)
    image_url = models.CharField(max_length=250 ,blank=True)
    image_url_replaced = models.CharField(max_length=250, default='')
    position = models.CharField(max_length=10, choices=settings.TEXT_AND_IMAGE_CONTENT_POSITIONS)
    
    baselayout = "content/text_and_image/section.html"
    
    class Meta:
        abstract = True
        verbose_name = _('text and image content')
        verbose_name_plural = _('text and image contents')
    
    def get_extra_context(self):
        text_width = settings.NEWSLETTER_CONTENT_WIDTH if self.position == 'top' else (settings.NEWSLETTER_CONTENT_WIDTH - 20 - settings.TEXT_AND_IMAGE_CONTENT_IMAGE_WIDTH_SIDE)
        return {
            'image_width': self.image_width,
            'image_height': self.image_height,
            'text_width': text_width,
        }

    def get_image_url(self, context=None):
        """
        Gives the repalced url back, if no mail is present it gives instead
        the original url.
        """
        if context is None:
            return self.image_url
        template = Template(self.image_url_replaced)
        return template.render(context)
            
    def replace_links(self, job):
        super(TextWithImageNewsletterContent, self).replace_links(job)
        if not is_link(self.image_url, self.image_url_replaced):
            self.image_url_replaced = job.add_link(self.image_url)
            self.save()
            
    def save(self, *args, **kwargs):
        image_width = settings.NEWSLETTER_CONTENT_WIDTH if self.position == 'top' else settings.TEXT_AND_IMAGE_CONTENT_IMAGE_WIDTH_SIDE
        im=Image.open(self.image_original.file.path)
        im.thumbnail((image_width, 1000), Image.ANTIALIAS)
        img_temp = files.temp.NamedTemporaryFile()
        im.save(img_temp,'jpeg', quality=settings.JPEG_QUALITY, optimize=True)
        img_temp.flush()
        self.image_thumb.save(os.path.split(self.image_original.file.name)[1], files.File(img_temp), save=False)
        super(TextWithImageNewsletterContent, self).save(*args, **kwargs)
        
    



from django import forms
from django.contrib import admin
from django.contrib.admin.util import unquote
from django.conf.urls.defaults import patterns, url
from django.contrib.contenttypes import generic
from django.core import mail
from django.core.context_processors import csrf
from django.core.urlresolvers import reverse
from django.db import models
from django.http import HttpResponseRedirect
from django.shortcuts import render_to_response
from django.utils import translation
from django.utils.translation import ugettext_lazy as _

from pennyblack import settings

import datetime

#-----------------------------------------------------------------------------
# Job
#-----------------------------------------------------------------------------
class Job(models.Model):
    """A bunch of participants wich receive a newsletter"""
    newsletter = models.ForeignKey('pennyblack.Newsletter', related_name="jobs", null=True)
    status = models.IntegerField(choices=settings.JOB_STATUS, default=1)
    date_created = models.DateTimeField(verbose_name=_("created"), default=datetime.datetime.now())
    date_deliver_start = models.DateTimeField(blank=True, null=True, verbose_name=_("started delivering"), default=None)
    date_deliver_finished = models.DateTimeField(blank=True, null=True, verbose_name=_("finished delivering"), default=None)

    content_type = models.ForeignKey('contenttypes.ContentType', null=True, blank=True)
    object_id = models.PositiveIntegerField(null=True, blank=True)
    group_object = generic.GenericForeignKey('content_type', 'object_id')
    collection = models.TextField(blank=True)
    
    #ga tracking
    utm_campaign = models.SlugField(verbose_name=_("utm campaign"), blank=True)
    
    
    class Meta:
        ordering = ('-date_created',)
        verbose_name = _("newsletter delivery task")
        verbose_name_plural = _("newsletter delivery tasks")
        app_label = 'pennyblack'
        
        
    def __unicode__(self):
        return (self.newsletter.subject if self.newsletter is not None else "unasigned delivery task")
    
    def delete(self, *args, **kwargs):
        """
        If the job refers to a inactive Newsletter delete it.
        """
        if self.newsletter.active == False:
            self.newsletter.delete()
        super(Job, self).delete(*args, **kwargs)
    
    @property
    def count_mails_total(self):
        return self.mails.count()
    
    @property
    def count_mails_sent(self):
        return self.mails.filter(sent=True).count()

    @property
    def percentage_mails_sent(self):
        if self.count_mails_total == 0:
            return 0
        return round(float(self.count_mails_sent)/float(self.count_mails_total) * 100, 1)
    
    @property
    def count_mails_viewed(self):
        return self.mails.exclude(viewed=None).count()
    
    @property
    def count_mails_delivered(self):
        return self.count_mails_sent - self.count_mails_bounced

    @property
    def percentage_mails_viewed(self):
        if self.count_mails_delivered == 0:
            return 0
        return round(float(self.count_mails_viewed)/self.count_mails_delivered * 100, 1)
    
    @property
    def count_mails_bounced(self):
        return self.mails.filter(bounced=True).count()

    @property
    def count_mails_clicked(self):
        return self.mails.filter(clicks__isnull=False).count()
    
    @property
    def percentage_mails_clicked(self):
        if self.count_mails_delivered == 0:
            return 0
        return round(float(self.count_mails_clicked)/float(self.count_mails_delivered) * 100, 1)

    @property
    def percentage_mails_bounced(self):
        if self.count_mails_sent == 0:
            return 0
        return round(float(self.count_mails_bounced)/float(self.count_mails_sent) * 100, 1)

    # fields
    def field_mails_sent(self):
        return self.count_mails_sent
    field_mails_sent.short_description = _('# of mails sent')
    
    def field_opening_rate(self):
        return '%s%%' % self.percentage_mails_viewed
    field_opening_rate.short_description = _('opening rate')
    
    def field_mails_total(self):
        return self.count_mails_total
    field_mails_total.short_description = _('# of mails')
    

    def can_send(self):
        """
        Is used to determine if a send button should be displayed.
        """
        if not self.status in settings.JOB_STATUS_CAN_SEND:
            return False
        return self.is_valid()

    def is_valid(self):
        if self.newsletter == None or not self.newsletter.is_valid():
            return False
        return True
    
    def create_mails(self, queryset):
        """
        Create mails for every NewsletterReceiverMixin in queryset.
        """
        if hasattr(queryset, 'iterator') and callable(queryset.iterator):
            for receiver in queryset.iterator():
                self.create_mail(receiver)
        else:
            for receiver in queryset:
                self.create_mail(receiver)
            
    def create_mail(self, receiver):
        """
        Creates a single mail. This is also used in workflow mail send process.
        receiver has to implement all the methods from NewsletterReceiverMixin
        """
        return self.mails.create(person=receiver)
        
    
    def add_link(self, link, identifier=''):
        """
        Adds a link and returns a replacement link
        """
        if identifier != '':
            try:
                return self.links.get(identifier=identifier)
            except self.links.model.DoesNotExist:
                return self.links.create(link_target='', identifier=identifier)
        # clean link from htmlentities
        for old, new in (('&amp;','&'),('&lt;','<'),('&gt;','>'),('&quot;','"')):
            link = link.replace(old, new)
        link = self.links.create(link_target=link)
        link.save()
        return '{{base_url}}' + reverse('pennyblack.redirect_link', kwargs={'mail_hash':'{{mail.mail_hash}}','link_hash':link.link_hash}).replace('%7B','{').replace('%7D','}')
    
    def send(self):
        """
        Sends every pending e-mail in the job.
        """
        self.newsletter = self.newsletter.create_snapshot()
        self.newsletter.replace_links(self)
        self.newsletter.prepare_to_send()
        self.status = 21
        self.date_deliver_start = datetime.datetime.now()
        self.save()
        try:
            translation.activate(self.newsletter.language)
            connection = mail.get_connection()
            connection.open()
            for newsletter_mail in self.mails.filter(sent=False).iterator():
                connection.send_messages([newsletter_mail.get_message()])
                newsletter_mail.mark_sent()
            connection.close()
        except:
            self.status = 41
            raise
        else:
            self.status = 31
            self.date_deliver_finished = datetime.datetime.now()
        self.save()

class JobAdminForm(forms.ModelForm):
    from pennyblack.models.newsletter import Newsletter
    newsletter = forms.ModelChoiceField(queryset=Newsletter.objects.massmail())

class JobAdmin(admin.ModelAdmin):
    from pennyblack.models.link import LinkInline
    from pennyblack.models.mail import MailInline
    
    date_hierarchy = 'date_deliver_start'
    actions = None
    list_display = ('newsletter', 'group_object', 'status', 'field_mails_total', 'field_mails_sent', 'date_created')
    list_filter   = ('status', 'newsletter',)
    fields = ('newsletter', 'collection', 'status', 'group_object', 'field_mails_total', 'field_mails_sent', 'date_deliver_start', 'date_deliver_finished', 'utm_campaign')
    readonly_fields = ('collection', 'status', 'group_object', 'field_mails_total', 'field_mails_sent', 'date_deliver_start', 'date_deliver_finished',)    
    inlines = (LinkInline, MailInline,)
    massmail_form = JobAdminForm
    
    def get_form(self, request, obj=None, **kwargs):
        if obj and obj.status in settings.JOB_STATUS_CAN_EDIT:
            kwargs['form'] = self.massmail_form
        return super(JobAdmin, self).get_form(request, obj, **kwargs)
    
    def get_readonly_fields(self, request, obj=None):
        if obj and obj.status in settings.JOB_STATUS_CAN_EDIT:
            return self.readonly_fields
        else:
            return self.readonly_fields + ('newsletter',)
            
    def change_view(self, request, object_id, extra_context={}):
        obj = self.get_object(request, unquote(object_id))
        extra_context['can_send']=obj.can_send()
        request._pennyblack_job_obj = obj # add object to request for the mail inline
        return super(JobAdmin, self).change_view(request, object_id, extra_context=extra_context)

    def send_newsletter_view(self,request, object_id):
        obj = self.get_object(request, unquote(object_id))
        if request.method == 'POST' and request.POST.has_key("_send"):
            obj.status = 11
            obj.save()
            self.message_user(request, _("Newsletter has been marked for delivery."))
        return HttpResponseRedirect(reverse('admin:%s_%s_changelist' %(self.model._meta.app_label,  self.model._meta.module_name)))

    def response_change(self, request, obj):
        """
        Determines the HttpResponse for the change_view stage.
        """
        if request.POST.has_key("_send_prepare"):
            context = {
                'object':obj,
                'opts':self.model._meta,
                'app_label':self.model._meta.app_label,
            }
            context.update(csrf(request))
            return render_to_response(
                'admin/pennyblack/job/send_confirmation.html', context)
        return super(JobAdmin,self).response_change(request, obj)

    def get_urls(self):
        urls = super(JobAdmin, self).get_urls()
        info = self.model._meta.app_label, self.model._meta.module_name
        my_urls = patterns('',
            url(r'^(?P<object_id>\d+)/send/$', self.admin_site.admin_view(self.send_newsletter_view), name=('%s_%s_send' % info)),
        )
        return my_urls + urls
    
    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False
    

#-----------------------------------------------------------------------------
# Job
#-----------------------------------------------------------------------------
class JobStatistic(Job):
    class Meta:
        proxy = True
        verbose_name = _("statistic")
        verbose_name_plural = _("statistics")
        app_label = 'pennyblack'
        
class JobStatisticAdmin(admin.ModelAdmin):
    date_hierarchy = 'date_deliver_start'
    actions = None
    list_display = ('newsletter', 'group_object', 'field_mails_total', 'field_mails_sent', 'field_opening_rate', 'date_created')
    # list_filter   = ('status', 'newsletter',)
    fields = ('newsletter', 'collection', 'group_object', 'date_deliver_start', 'date_deliver_finished', 'utm_campaign')
    readonly_fields = ('newsletter', 'collection', 'group_object', 'date_deliver_start', 'date_deliver_finished', 'utm_campaign')    

    def queryset(self, request):
        return self.model.objects.exclude(status=1)

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

    def get_graph_data(self,obj):
        date_start = obj.date_deliver_start.replace(minute=0,second=0,microsecond=0)
        try:
            from django.utils import timezone
        except ImportError:
            now = datetime.datetime.now()
        else:
            now = timezone.now()
        opened_serie = []
        for i in range(336):
            t = date_start + datetime.timedelta(hours=i)
            count_opened = obj.mails.exclude(viewed=None).filter(viewed__lt=t).count()
            opened_serie.append('[%s000,%s]' % (t.strftime('%s'),count_opened))
            if t > now:
                break
        return {
            'opened_serie': ','.join(opened_serie),
        }
         
    def change_view(self, request, object_id, extra_context={}):
        obj = self.get_object(request, unquote(object_id))
        graph_data = self.get_graph_data(obj)
        extra_context.update(graph_data)
        return super(JobStatisticAdmin, self).change_view(request, object_id, extra_context=extra_context)



from django.core.validators import email_re

from pennyblack.module.subscriber.models import NewsletterSubscriber, SubscriberGroup

def add_subscriber(email,groups=[], **kwargs):
    """
    Adds a subscriber to the given groups
    """
    if not email_re.match(email):
        return False
    subscriber = NewsletterSubscriber.objects.get_or_add(email, **kwargs)
    for group_name in groups:
        group = SubscriberGroup.objects.get_or_add(group_name)
        if group not in subscriber.groups.all():
            subscriber.groups.add(group)
    return subscriber


from django.core.management.base import BaseCommand, CommandError
from pennyblack.models import Sender

class Command(BaseCommand):
    args = ''
    help = 'Gets all Bounce emails'

    def handle(self, *args, **options):
        senders = Sender.objects.filter(get_bounce_emails=True)
        for sender in senders:
            sender.get_mail()



from django.contrib import admin
from django.core.urlresolvers import resolve
from django.db import models
from django.template import Context, Template, TemplateSyntaxError
from django.utils.translation import ugettext_lazy as _

import datetime
import hashlib
import random

#-----------------------------------------------------------------------------
# Link
#-----------------------------------------------------------------------------

def is_link(link_original, link_replaced):
    """
    Checks if link_replaced resolves to link_original
    """
    from pennyblack.models import Link
    if link_replaced == '':
        return False
    # try to find the link and compare it to the header_url but
    # replace the link if something goes wrong
    try:
        link_hash = resolve(link_replaced[len('{{base_url}}'):])[2]['link_hash']
        link = Link.objects.get(link_hash=link_hash)
        if link.link_target == link_original:
            return True
    except:
        pass
    return False

def check_if_redirect_url(url):
    """
    Checks if the url is a redirect url
    """
    if '{{base_url}}' == url[:len('{{base_url}}')]:
        try:
            result = resolve(url[len('{{base_url}}'):])
            if result[0].func_name == 'redirect_link':
                return True
        except:
            pass
    return False

class Link(models.Model):
    job = models.ForeignKey('pennyblack.Job', related_name='links')
    identifier = models.CharField(max_length=100, default='')
    link_hash = models.CharField(max_length=32, verbose_name=_("link hash"), db_index=True, blank=True)
    link_target = models.CharField(verbose_name=_("address"), max_length=500, default='')
    
    class Meta:
        verbose_name = _('link')
        verbose_name_plural = _('links')
        app_label = 'pennyblack'

    def __unicode__(self):
        return self.link_target
        
    def click_count(self):
        """
        Returns the total click count.
        """
        return self.clicks.count()
    click_count.short_description = 'Click count'
    
    def click(self,mail):
        """
        Creates a LinkClick and returns the link target
        """
        click = self.clicks.create(mail=mail)
        return self.get_target(mail)
    
    def get_target(self, mail):
        """
        gets the link target by evaluating the string using the email content
        """
        from pennyblack.models import Newsletter
        if self.identifier != '':
            return Newsletter.get_view_link(self.identifier)
        template = Template(self.link_target)
        return template.render(Context(mail.get_context()))

    def save(self, **kwargs):
        if self.link_hash == u'':
            self.link_hash = hashlib.md5(str(self.id)+str(random.random())).hexdigest()
        super(Link, self).save(**kwargs)
        
class LinkClick(models.Model):
    link = models.ForeignKey('pennyblack.Link', related_name='clicks')
    mail = models.ForeignKey('pennyblack.Mail', related_name='clicks')
    date = models.DateTimeField(default=datetime.datetime.now())
    
    class Meta:
        app_label = 'pennyblack'
        
class LinkInline(admin.TabularInline):
    model = Link
    max_num = 0
    can_delete = False
    fields = ('link_target', 'link_hash',)
    readonly_fields = ('link_hash',)
    
    def queryset(self, request):
        """
        Don't show links with identifier because they aren't changable.
        """
        queryset = super(LinkInline, self).queryset(request)
        return queryset.filter(identifier='')



from django import template
from django.core.urlresolvers import reverse

register = template.Library()

class NewsletterstyleNode(template.Node):
    def __init__(self, key, style, request):
        self.key = key
        self.style = style
        self.request_var = template.Variable(request)
        
    def render(self, context):
        request = self.request_var.resolve(context)
        if not hasattr(request, '_pennyblack_newsletterstyle'):
            setattr(request, '_pennyblack_newsletterstyle', dict())
        request._pennyblack_newsletterstyle[str(self.key)] = self.style.render(context)
        return ''

def newsletterstyle(parser, token):
    """
    Stores the content in your context
    {% newsletterstyle request text_only_content %}
    """
    bits = list(token.split_contents())
    if len(bits) != 3 :
        raise template.TemplateSyntaxError("%r expected format is 'newsletterstyle request stylename'" %
            bits[0])
    request = bits[1]
    key = parser.compile_filter(bits[2])
    style = parser.parse(('endnewsletterstyle',))
    parser.delete_first_token()
    return NewsletterstyleNode(key, style, request)
newsletterstyle = register.tag(newsletterstyle)

class NewsletterGetStyleNode(template.Node):
    def __init__(self, key, request):
        self.key = key
        self.request_var = template.Variable(request)
    
    def render(self, context):
        request = self.request_var.resolve(context)
        if not hasattr(request, '_pennyblack_newsletterstyle'):
            return ''
        if str(self.key) not in request._pennyblack_newsletterstyle.keys():
            return ''
        return request._pennyblack_newsletterstyle[str(self.key)]
        
def get_newsletterstyle(parser, token):
    """
    Loads a stored style and renders it
    """
    bits = list(token.split_contents())
    if len(bits) != 3 :
        raise template.TemplateSyntaxError("%r expected format is 'newsletterstyle request stylename'" %
            bits[0])
    request = bits[1]
    key = parser.compile_filter(bits[2])
    return NewsletterGetStyleNode(key, request)
get_newsletterstyle = register.tag(get_newsletterstyle)


class NewsletterHeaderImageNode(template.Node):
    def __init__(self, extra_args={}):
        self.extra_args = extra_args
        
    def render(self, context):
        newsletter = context['newsletter']
        if context['webview']:
            if 'mail' in context:
                mail = context['mail']
                header_url = mail.get_header_url()
            else:
                header_url = ''
            header_image = newsletter.header_image.get_absolute_url()
        else:
            mail = context['mail']
            header_url = mail.get_header_url()
            header_image = newsletter.get_base_url() + reverse('pennyblack.ping', kwargs={'mail_hash':mail.mail_hash, 'filename':newsletter.header_image})
        return """<a href="%s" target="_blank"><img src="%s" border="0" %s/></a>""" % (header_url, header_image, ' '.join(self.extra_args))

@register.tag
def header_image(parser, token):
    """
    Renders the header image with tracking any extra args are given to the img tag
    
    {% header_image alt="Header" width="283" height="157" align="left" vspace="0" hspace="0" border="0" %}
    
    renders
    
    <a href="" target="_blank"><img src="link to the image" border="0" alt="Header" width="283" height="157" align="left" vspace="0" hspace="0" border="0"/></a>
    """
    bits = list(token.split_contents())
    extra_args = []
    for bit in bits:
        splitted = bit.split('=')
        if len(splitted) == 2:
            extra_args.append(bit)
    return NewsletterHeaderImageNode(extra_args=extra_args)

class NewsletterLinkUrlNode(template.Node):
    def __init__(self, identifier=None):
        self.identifier=identifier
        
    def render(self, context):
        from pennyblack.models import Link, Newsletter
        if 'mail' not in context:
            return u'#'
        mail = context['mail']
        newsletter = mail.job.newsletter
        if newsletter.is_workflow():
            job = newsletter.get_default_job()
        else:
            job = mail.job
        try:
            link = job.links.get(identifier=self.identifier)
        except job.links.model.DoesNotExist:
            link = Newsletter.add_view_link_to_job(self.identifier, job)
        return context['base_url'] + reverse('pennyblack.redirect_link', args=(mail.mail_hash, link.link_hash))

@register.tag
def link_url(parser, token):
    """
    Renders a link wich is provided by the group object.
    """
    bits = list(token.split_contents())
    if len(bits) != 2:
        raise template.TemplateSyntaxError("%r expected format is 'link_url url_identifier'" %
            bits[0])
    return NewsletterLinkUrlNode(identifier=bits[1])

class ContentImageUrlNode(template.Node):
    def render(self, context):
        if 'mail' in context:
            return context['content'].get_image_url(context=context)
        return context['content'].get_image_url()

@register.tag
def content_image_url(parser, token):
    """
    Renders the link of the given content.
    """
    return ContentImageUrlNode()



# coding=utf-8
from pennyblack.models.newsletter import Newsletter
from pennyblack.models.job import Job
from pennyblack.models.link import Link, LinkClick
from pennyblack.models.mail import Mail
from pennyblack.models.sender import Sender

__all__ = ('Newsletter','Job', 'Link', 'LinkClick', 'Mail', 'Sender')



#! /usr/bin/env python
import os
from setuptools import setup

import pennyblack
setup(
    name='pennyblack',
    version = pennyblack.__version__,
    description = 'django based newsletter toolkit',
    long_description=open(os.path.join(os.path.dirname(__file__), 'README.rst')).read(),
    author = 'Marc Egli',
    author_email = 'egli@allink.ch',
    url = 'http://github.com/allink/pennyblack/',
    license='BSD License',
    platforms=['OS Independent'],
    packages=[
        'pennyblack',
        'pennyblack.content',
        'pennyblack.management',
        'pennyblack.management.commands',
        'pennyblack.models',
        'pennyblack.module',
        'pennyblack.module.subscriber',
        'pennyblack.templatetags',
    ],
    # package_data={'pennyblack':'templates/*.html'},
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Communications :: Email',
    ],
    requires=[
        'FeinCMS(>=1.3.0)',
        'Django(>=1.3)',
        'pydns',
        'pyspf',
        'pil',
    ],
    include_package_data=True,    
)


from django.core.management.base import BaseCommand, CommandError
from pennyblack.models import Job
from pennyblack import settings

class Command(BaseCommand):
    args = ''
    help = 'Sends all pending Newsletters'

    def handle(self, *args, **options):
        pending_jobs = Job.objects.filter(status__in=settings.JOB_STATUS_PENDING)
        for job in pending_jobs:
            job.send()
            print str(job)+ ' sent'


from django.conf.urls.defaults import *

urlpatterns = patterns('',
    url(r'^link/(?P<mail_hash>[^/]+)/(?P<link_hash>[a-z0-9]+)/$', 'pennyblack.views.redirect_link', name='pennyblack.redirect_link'),
    url(r'^proxy/(?P<mail_hash>[^/]+)/(?P<link_hash>[a-z0-9]+)/$', 'pennyblack.views.proxy', name='pennyblack.proxy'),
    url(r'^view/(?P<mail_hash>\w+)', 'pennyblack.views.view', name='pennyblack.view'),
    url(r'^ping/(?P<mail_hash>\w*)/(?P<filename>.*)$', 'pennyblack.views.ping', name='pennyblack.ping'),    
)



# coding=utf-8
import exceptions

from django.conf.urls.defaults import patterns, url
from django.core.exceptions import ImproperlyConfigured
from django.contrib import admin
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.db.models import signals
from django.utils import translation
from django.utils.translation import ugettext_lazy as _

from feincms.admin import editor
from feincms.admin import item_editor
from feincms.management.checker import check_database_schema
from feincms.models import Base
from feincms.utils import copy_model_instance

from pennyblack import settings

#-----------------------------------------------------------------------------
# Newsletter
#-----------------------------------------------------------------------------
class NewsletterManager(models.Manager):
    def active(self):
        """
        Filters all active newsletters
        """
        return self.filter(active=True)
    
    def massmail(self):
        """
        Filters all newsletter avaiable for massmailing
        """
        return self.active().filter(newsletter_type__in=settings.NEWSLETTER_TYPE_MASSMAIL)
        
    def workflow(self):
        """
        Filters all newsletter avaiable in a workflow eg. signupmail
        """
        return self.active().filter(newsletter_type__in=settings.NEWSLETTER_TYPE_WORKFLOW)
    
    def get_workflow_newsletter_by_name(self, name):
        """
        Tries to get a newsletter with the given name. First it tries to find
        one where the language matches the active language, later it tries to
        find one with the default language and if it doesn't find one it tries
        to get any newsletter with the given name before giving up.
        """
        try:
            return self.workflow().get(name__iexact=name, language=translation.get_language())
        except models.ObjectDoesNotExist:
            pass
        try:
            return self.workflow().get(name__iexact=name, language=settings.LANGUAGE_CODE)
        except models.ObjectDoesNotExist:
            pass
        try:
            return self.workflow().filter(name__iexact=name)[0]
        except:
            return None

class Newsletter(Base):
    """
    A newsletter with subject and content
    can contain multiple jobs with mails to send
    """
    name = models.CharField(verbose_name=_("name"), help_text=_("Is only to describe the newsletter."), max_length=100)
    active = models.BooleanField(default=True)
    newsletter_type = models.IntegerField(choices=settings.NEWSLETTER_TYPE,
        verbose_name=_("Type"), help_text=_("Can only be changed during creation."))
    sender = models.ForeignKey('pennyblack.Sender', verbose_name=_("sender"))
    subject = models.CharField(verbose_name=_("subject"), max_length=250)
    reply_email = models.EmailField(verbose_name=_("reply-to") ,blank=True)
    language = models.CharField(max_length=6, verbose_name=_("language"), choices=settings.LANGUAGES)
    header_image = models.ForeignKey('medialibrary.MediaFile', verbose_name=_("header image"))
    header_url = models.URLField()
    header_url_replaced = models.CharField(max_length=250, default='')
    site = models.ForeignKey('sites.Site', verbose_name=_("site"))    
    #ga tracking
    utm_source = models.SlugField(verbose_name=_("utm Source"), default="newsletter")
    utm_medium = models.SlugField(verbose_name=_("utm Medium"), default="cpc")
    
    objects = NewsletterManager()

    class Meta:
        ordering = ('subject',)
        verbose_name = _("Newsletter")
        verbose_name_plural = _("Newsletters")
        app_label = 'pennyblack'
            
    def __unicode__(self):
        return u'%s %s' % (self.name, self.language)
    
    def is_valid(self):
        """
        Checks if the newsletter is valid. A newsletter needs to have a
        subject to be valid.
        """
        if self.subject == '':
            return False
        # todo: check if email is valid
        return True
    
    def create_snapshot(self):
        """
        Makes a copy of itselve with all the content and returns the copy.
        """
        snapshot = copy_model_instance(self, exclude=('id',))
        snapshot.active = False
        snapshot.save()
        snapshot.copy_content_from(self)
        return snapshot
        
    def get_base_url(self):
        return "http://" + self.site.domain
        
    def replace_links(self, job):
        """
        Searches al links in content sections and replaces them with a link to
        the link tracking view.
        It also generates the header_url_replaced which is the same but for
        the header url.
        """
        from pennyblack.models.link import is_link
        if self.is_workflow():
            default_job = self.get_default_job()
        else:
            default_job = job
        for cls in self._feincms_content_types:
            for content in cls.objects.filter(parent=self):
                content.replace_links(default_job)
                content.save()
        if not is_link(self.header_url, self.header_url_replaced):
            self.header_url_replaced = default_job.add_link(self.header_url)
            self.save()
        if job.group_object and hasattr(job.group_object,'get_extra_links'):
            from exceptions import DeprecationWarning
            raise DeprecationWarning("get_extra_links is deprecated and will no longer work")
        
    def prepare_to_send(self):
        """
        Last hook before the newsletter is sent
        """
        for cls in self._feincms_content_types:
            for content in cls.objects.filter(parent=self):
                if hasattr(content, 'prepare_to_send'):
                    content.prepare_to_send()
    
    def get_default_job(self):
        """
        Tries to get the default job. If no default job exists it creates one.
        This is only used in workflow newsletters.
        """
        try:
            return self.jobs.get(content_type=None)
        except models.ObjectDoesNotExist:
            return self.jobs.create(status=32)            
    
    def is_workflow(self):
        """
        Returns True if it's type is a workflow newsletter.
        """
        return self.newsletter_type in settings.NEWSLETTER_TYPE_WORKFLOW 

    def send(self, person, group=None):
        """
        Sends this newsletter to "person" with optional "group".
        This works only with newsletters which are workflow newsletters.
        """
        if not self.is_workflow():
            raise exceptions.AttributeError('only newsletters with type workflow can be sent')
        # search newsletter job wich hash the same group or create it if it doesn't exist
        try:
            if group:
                ctype = ContentType.objects.get_for_model(group)
                job = self.jobs.get(content_type__pk=ctype.id, object_id=group.id)
            else:
                job = self.jobs.get(content_type=None)
        except models.ObjectDoesNotExist:
            if group:
                kw = {'group_object':group}
            else:
                kw = {}
            job=self.jobs.create(status=32, **kw) # 32=readonly
        self.replace_links(job)
        self.prepare_to_send()
        mail = job.create_mail(person)
        try:
            message = mail.get_message()
            message.send()
        except:
            raise
        else:
            mail.mark_sent()
    
    _view_links = {}
    
    @classmethod
    def register_view_link(cls, identifier, view):
        """
        Register a new view link
        Newsletter.register_view_link('my_identifier',view_function)
        """
        if identifier in cls._view_links.keys():
            return
        cls._view_links[identifier] = view
    
    @classmethod
    def add_view_link_to_job(cls, identifier,job):
        if identifier not in cls._view_links.keys():
            raise ImproperlyConfigured("no view with identifier '%s' found" % identifier)
        return job.add_link(cls._view_links[identifier], identifier=identifier)
    
    @classmethod
    def get_view_link(cls, identifier):
        if identifier not in cls._view_links.keys():
            raise ImproperlyConfigured("no view with identifier '%s' found" % identifier)
        return cls._view_links[identifier]
        
            
Newsletter.__module__ = 'pennyblack.models'    
signals.post_syncdb.connect(check_database_schema(Newsletter, __name__), weak=False)

class NewsletterAdmin(editor.ItemEditor, admin.ModelAdmin):
    list_display = ('name', 'subject', 'language', 'newsletter_type')
    raw_id_fields = ('header_image',)
    fieldsets = (
        (None, {
            'fields': ['name', 'subject', 'sender', 'reply_email', 'template_key'],
        }),
        (_('Other options'), {
            'classes': ['collapse',],
            'fields': ('newsletter_type', 'language', 'utm_source','utm_medium', 'header_image', 'header_url', 'site'),
        }),
        item_editor.FEINCMS_CONTENT_FIELDSET,
    )
    exclude = ('header_url_replaced',)

    def get_readonly_fields(self, request, obj=None):
        if obj:
            return self.readonly_fields + ('newsletter_type',)
        return self.readonly_fields

    def queryset(self, request):
        return self.model.objects.active()

    def get_urls(self):
        urls = super(NewsletterAdmin, self).get_urls()
        my_urls = patterns('',
            (r'^(?P<newsletter_id>\d+)/preview/$', 'pennyblack.views.preview')
        )
        return my_urls + urls



from django.shortcuts import render_to_response

def unsubscribe(request, person, group_object=None):
    """
    Unsubscribe view
    """
    context = {
        'person': person,
        'group_object': group_object,
    }
    if request.GET.get('unsubscribe', False):
        person.unsubscribe()
        context.update({'done':True})
    return render_to_response('pennyblack/subscriber/unsubscribe.html', context)